﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;

namespace TestMultithread
{
    public partial class Form1 : Form
    {
        // pour dessiner à l'écran
        Graphics canvas;

        // liste de tous les threads
        List<Thread> LThreads = new List<Thread>();

        // largeur de la zone carrée
        int LargZonePix = 50;           
        
        // liste des zones carré à traiter
        ConcurrentBag<Point> JobList = new ConcurrentBag<Point>();  

        public Form1()
        {
            InitializeComponent();
            canvas = pictureBox1.CreateGraphics();
        }

        // lorsque vous cliquez sur le bouton dans la fenêtre
        private void button1_Click(object sender, EventArgs e)
        {
            // crée la liste des zones à afficher
            int LargAff = pictureBox1.Width;
            int HautAff = pictureBox1.Height;
            for (int x = 0; x < LargAff; x += LargZonePix)
                for (int y = 0; y < HautAff; y += LargZonePix)
                    JobList.Add(new Point(x, y));

            // crée et lance le pool de threads
            canvas.FillRectangle(Brushes.LightPink,0,0,2000,2000);
            for (int i = 0; i < 4; i++)  // 4: nb de threads
            {
                int idThread = i; // capture correctement la valeur de i pour le délégué ci-dessous
                Thread T = new Thread(delegate () { FntThread(idThread); });
                LThreads.Add(T);
                T.Start();        // demarre le thread enfant
            }
        }

        delegate void MonDelegue(Point P, Bitmap B);

        // fonction appelée dans le thread principal suite à l'envoi d'un évènement 
        // par un thread enfant grâce à la méthode invoke

        private void DrawInMainThread(Point P, Bitmap B)
        {
            canvas.DrawImage(B, P);
        }

        Brush[] LBrushes = { Brushes.Black, Brushes.DarkGray, Brushes.LightGray, Brushes.White };
        int[] Temps = { 500, 1000, 1500, 2000 };

        // methode déclenchée par chaque thread
        // le code ci-dessous s'exécute dans les threads enfants

        private void FntThread(int idThread)
        {
            Brush MaCouleur = LBrushes[idThread];
            int MonTemps    = Temps[idThread];

            Point CoordZone;

            // capture une zone dans la liste des zones à traiter
            while (JobList.TryTake(out CoordZone))  
            {
                Thread.Sleep(MonTemps);
                // crée un bitmap local correspondant à la zone à dessiner
                Bitmap B = new Bitmap(LargZonePix, LargZonePix);
                Graphics G = Graphics.FromImage(B);
                // remplit la zone de la couleur du thread
                G.FillRectangle(MaCouleur, 0, 0, LargZonePix, LargZonePix);

                // renvoi les infos suffisantes dans un évènement pour que
                // le thread principal puisse dessiner la région au bon endroit
                var d = new MonDelegue(DrawInMainThread);
                pictureBox1.Invoke(d, new object[] { CoordZone, B });
            }
        }

        // arrête les threads si fermeture de la fenêtre
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            foreach (Thread T in LThreads)
                T.Abort();
        }
    }
}
